package su.a71.new_soviet.blocks;

import net.minecraft.block.*;
import net.minecraft.entity.player.PlayerEntity;
import net.minecraft.fluid.FluidState;
import net.minecraft.fluid.Fluids;
import net.minecraft.item.BucketItem;
import net.minecraft.item.DyeItem;
import net.minecraft.sound.SoundCategory;
import net.minecraft.sound.SoundEvents;
import net.minecraft.state.StateManager;
import net.minecraft.state.property.BooleanProperty;
import net.minecraft.state.property.IntProperty;
import net.minecraft.state.property.Properties;
import net.minecraft.util.ActionResult;
import net.minecraft.util.DyeColor;
import net.minecraft.util.Hand;
import net.minecraft.util.hit.BlockHitResult;
import net.minecraft.util.math.BlockPos;
import net.minecraft.util.math.Direction;
import net.minecraft.util.shape.VoxelShape;
import net.minecraft.world.BlockView;
import net.minecraft.world.World;
import net.minecraft.world.WorldAccess;

// TODO: BLOCKTAGS! LOOTABLES!
public class BoundaryMarkerBlock extends Block implements Waterloggable {
    public static final BooleanProperty WATERLOGGED;
    public static final IntProperty COLOUR;
    public static final VoxelShape SHAPE;

    public BoundaryMarkerBlock(Settings settings) {
        super(settings);
        setDefaultState(getDefaultState()
                .with(WATERLOGGED, false)
                .with(COLOUR, 0));
    }

    @Override
    protected void appendProperties(StateManager.Builder<Block, BlockState> builder) {
        builder.add(WATERLOGGED, COLOUR);
    }

    public VoxelShape getOutlineShape(BlockState state, BlockView world, BlockPos pos, ShapeContext context) {
        return SHAPE;
    }

    @Override
    public ActionResult onUse(BlockState state, World world, BlockPos pos, PlayerEntity player, Hand hand, BlockHitResult hit) {
        if (player.getInventory().getMainHandStack().getItem() instanceof DyeItem) {
            if (!world.isClient()) {
                int id = ((DyeItem) player.getInventory().getMainHandStack().getItem()).getColor().getId();
                if (id == state.get(COLOUR))
                    return ActionResult.CONSUME;
                if (!player.isCreative()) {
                    player.getInventory().getMainHandStack().decrement(1);
                }
                world.setBlockState(pos, state.with(COLOUR, id));
                world.playSound((PlayerEntity)null, pos, SoundEvents.ITEM_DYE_USE, SoundCategory.BLOCKS, 1.0F, 1.0F);
            }
            return ActionResult.SUCCESS;
        }
        return super.onUse(state, world, pos, player, hand, hit);
    }

    @Override
    public BlockState getStateForNeighborUpdate(BlockState state, Direction direction, BlockState neighborState, WorldAccess world, BlockPos pos, BlockPos neighborPos) {
        if (state.get(WATERLOGGED)) {
            world.scheduleFluidTick(pos, Fluids.WATER, Fluids.WATER.getTickRate(world));
        }
        return super.getStateForNeighborUpdate(state, direction, neighborState, world, pos, neighborPos);
    }

    public FluidState getFluidState(BlockState state) {
        return state.get(WATERLOGGED) ? Fluids.WATER.getStill(false) : super.getFluidState(state);
    }

    static {
        WATERLOGGED = Properties.WATERLOGGED;
        SHAPE = Block.createCuboidShape(5, 0, 5, 11, 16, 11);
        COLOUR = IntProperty.of("border_colour", 0, 15);  // 0-16 = dyes
    }

    // DyeColor IDs with better colours
    public static int getColour(int id) {
        return switch (id) {
            case 0 -> 0xffffff;
            case 1 -> 0xb67134;
            case 2 -> 0xab4d8b;
            case 3 -> 0x398b9c;
            case 4 -> 0xb08732;
            case 5 -> 0x489d1b;
            case 6 -> 0xcf7a92;
            case 7 -> 0x353e42;
            case 8 -> 0x777771;
            case 9 -> 0x18797e;
            case 10 -> 0x813995;
            case 11 -> 0x354a85;
            case 12 -> 0x614332;
            case 13 -> 0x1b6031;
            case 14 -> 0x8b2030;
            case 15 -> 0x272228;
            default -> 0xffffff;
        };
    }
}
