package su.a71.new_soviet.blocks.lamps;

import net.minecraft.block.Block;
import net.minecraft.block.BlockState;
import net.minecraft.block.ShapeContext;
import net.minecraft.item.ItemPlacementContext;
import net.minecraft.state.StateManager;
import net.minecraft.state.property.DirectionProperty;
import net.minecraft.state.property.Properties;
import net.minecraft.util.BlockMirror;
import net.minecraft.util.BlockRotation;
import net.minecraft.util.math.BlockPos;
import net.minecraft.util.math.Direction;
import net.minecraft.util.shape.VoxelShape;
import net.minecraft.util.shape.VoxelShapes;
import net.minecraft.world.BlockView;
import net.minecraft.world.WorldView;

import su.a71.new_soviet.util.Shapes;
import java.util.List;

public class GoldenTableLampBlock extends LampBlock {
    public static final DirectionProperty FACING;
    public Shapes.HorizontalShapeLegacy SHAPE = new Shapes.HorizontalShapeLegacy(List.of(
            List.of(5.0, 0.0, 5.0, 11.0, 2.0, 11.0),
            List.of(7.0, 2.0, 7.0, 9.0, 4.0, 9.0),
            List.of(6.0, 4.0, 6.0, 10.0, 5.0, 10.0),
            List.of(3.0, 9.0, 3.0, 13.0, 13.0, 7.0)));

    public GoldenTableLampBlock(Settings settings) {
        super(settings, true, null);
        this.setDefaultState(this.stateManager.getDefaultState()
                .with(INVERTED, defaultLightState)
                .with(WATERLOGGED, false)
                .with(ON, false)
                .with(Properties.HORIZONTAL_FACING, Direction.NORTH));
    }

    @Override
    protected void appendProperties(StateManager.Builder<Block, BlockState> builder) {
        builder.add(Properties.HORIZONTAL_FACING);
        super.appendProperties(builder);
    }

    public VoxelShape getOutlineShape(BlockState state, BlockView world, BlockPos pos, ShapeContext context) {
        return switch (state.get(FACING)) {
            case NORTH -> SHAPE.north();
            case SOUTH -> SHAPE.south();
            case EAST -> SHAPE.east();
            case WEST -> SHAPE.west();
            default -> VoxelShapes.fullCube();
        };
    }


    public boolean canPlaceAt(BlockState state, WorldView world, BlockPos pos) {
        Direction direction = Direction.DOWN;
        return Block.sideCoversSmallSquare(world, pos.offset(direction), direction.getOpposite());
    }

    @Override
    public BlockState getPlacementState(ItemPlacementContext ctx) {
        return super.getPlacementState(ctx).with(Properties.HORIZONTAL_FACING, ctx.getHorizontalPlayerFacing().getOpposite());
    }

    public BlockState rotate(BlockState state, BlockRotation rotation) {
        return state.with(FACING, rotation.rotate(state.get(FACING)));
    }

    public BlockState mirror(BlockState state, BlockMirror mirror) {
        return state.rotate(mirror.getRotation(state.get(FACING)));
    }

    static {
        FACING = Properties.HORIZONTAL_FACING;
    }
}